/* Copyright (C) 2011,2012,2013,2014,2015,2016 Gustavo de ANdrade Barreto
  
      This program is free software: you can redistribute it and/or modify it under the terms of the 
      GNU General Public License as published by the Free Software Foundation, either version 3
      of the License, or any later version.
      This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
      without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
      See the GNU General Public License for more details.
      You should have received a copy of the GNU Affero General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
      Contact:
      Gustavo de Andrade Barreto
      gabarreto@usp.br
*/
// Créditos e Licenças utilizadas:
// EmonLibrary examples openenergymonitor.org, Licence GNU GPL V3
// ==============================================================================================================
// =====================================    V E R S Ã O   1 . 5     =============================================
// ==============================================================================================================
// Esta versão mostra no LCD: V, I, Fator de potência, Pot.Ativa, Pot.Aparente, Temperatura, Umidade, Data e Hora.
// Grava os dados em cartão SD, quando este estiver presente ao iniciar (com mensagem no LCD Ausente/Presente).
// Calibração pode ser conduzida no próprio instrumento a partir de medições externas. Fatores de calibração são
// memorizados em EEPROM do modulo do relógio. Se não puderem ser lidos, carrega defaults.
// Feitas várias otimizações do uso de memória a partir da versão 1.2 .
// Estado: Testado OK >> Produção.
// Compiled on Arduino 1.8.3
// ==============================================================================================================

// B I B L I O T E C A S
#include <EmonLib.h>       // Include Emon Library. Info: https://learn.openenergymonitor.org/ 
#include <LiquidCrystal.h> // biblioteca para o LCD
#include <DHT.h>           // biblioteca DHT (sensor de temperatura e umidade)
#include <Wire.h>          // biblioteca para comunicação I2C (hardware)
#include "RTClib.h"        // Biblioteca para o relógio DS3231 https://learn.adafruit.com/ds1307-real-time-clock-breakout-board-kit/overview que é uma derivação de http://news.jeelabs.org/code/
#include <SPI.h>           // Biblioteca para comunicação SPI, utilizado pelo Cartão SD
#include <SD.h>            // Biblioteca de manipulação de arquivos em cartão SD
#include <AT24CX.h>        // Biblioteca de manipulação de memória EEPROM. A AT24C32 existente no modulo do Relógio.Salvando dados de calibração. https://github.com/cyberp/AT24Cx

//Define qual pino do Arduino recebe a informação do botão da tampa frontal
#define BOTAO 2   //A4 - Pino do botão na tampa frontal Nota: Botão apertado = LOW

//Define o tipo de sensor DHT utilizado
#define DHTPIN 3  //A5 - Pino de dados do DHT11
#define DHTTYPE DHT11

/*-----( Declare Constants and Pin Numbers )-----*/
const int chipSelect = 10;    // Pin used for chipselect (CS)of the SD Card with SPI communications

/*-----( Declare Variables )-----*/
String sData = "";   // string for one "data to log" record. See reserve function bellow  
String sTimeStampD = "";   // string holding date-to-log. See reserve function bellow  
String sTimeStampT = "";   // string holding time-to-log. See reserve function bellow  
String svirgula = ";";
File logfile;             // the logging file object
unsigned int iCount;
unsigned long currentMillis;
unsigned long lastmillis;
unsigned long lseconds;
float tensao;           // Vrms
float Irms;             // Irms
float powerFActor;      // Fator de Potência
float realPower;        // Pot. Ativa
float apparentPower;    // Pot. Aparente
float tensaoMean;           // Vrms Médio no minuto
float IrmsMean;             // Irms Médio no minuto
float powerFActorMean;      // Fator de Potência Médio no minuto
float realPowerMean;        // Pot. Ativa Médio no minuto
float apparentPowerMean;    // Pot. Aparente Médio
double Vcal;                // calibration variables
double Ical;
double Fpcal;
double Vcaltemp;
double Icaltemp;
double Fpcaltemp;
float reaiskWh;        // Valor monetário do kWh
float reaiskWhtemp;    // Valor monetário do kWh
float AcumWatt;        // Watt acumulator
byte bCount = 0;
byte timeout1 = 0;            // timeout do menu calibração
byte timeout2 = 0;            // timeout do submenu da calibração
byte timeout5s = 0;          // timeout da leitura da temp e humidade (não deve ser lido em menos de 2s, fica impreciso)
byte menu = 0;                // qual tela aparece no LCD
byte pressionou;
byte subMenu = 0;           // qual submenu, 0=nenhum
bool BotaoAgora;            // botão apertado?
bool BotaoAntes;            // botão estava apertado?
bool Grava;
bool BlueTooth = true;
long tempo;
byte zero = 0;
uint16_t ano; byte mes; byte dia; byte hora; byte minu; byte seg;
float h;                    //para valor da umidade
float t;                    //para valor da temperatura

// =================================== O B J E T O S  (Instancias): 
DHT dht(DHTPIN, DHTTYPE);               // Cria instancia do sensor de umidade

//Pino 4 do LCD (RS) ligado ao pino 4 do Arduino
//Pino 6 do LCD (E) ligado ao pino 5 do Arduino
//Pino 11 do LCD (D4) ligado ao pino 6 do Arduino
//Pino 12 do LCD (D5) ligado ao pino 7 do Arduino
//Pino 13 do LCD (D6) ligado ao pino 8 do Arduino
//Pino 14 do LCD (D7) ligado ao pino 9 do Arduino
LiquidCrystal lcd(4, 5, 6, 7, 8, 9);    // Cria instância do LCD. Define os pinos que serão ligados ao LCD

EnergyMonitor emon1;                    // Cria instância do monitor de energia
RTC_DS3231 rtc;                         // Cria instância do relógio de tempo real DS3231
AT24CX mem;                             // Cria instância do chip de memória AT24C32

DateTime agora;   // estrutura da RTClib para conter a data e hora lida do relógio

// =======================================================    S E T U P   ====================
void setup()
{  
  Serial.begin(9600);    // inicia porta serial em 9600 bauds
  sData.reserve(13);  // reserve memory bytes for this string, making it static-like.
  sTimeStampD.reserve(11);    // reserve memory bytes for this string, making it static-like.
  sTimeStampT.reserve(9);     // reserve memory bytes for this string, making it static-like.  
  lcd.begin(16,2);           //Inicializa LCD
  lcd.clear();               //Limpa o LCD e posiciona na coluna 0, caractere 1
  lcd.print(F("Versao: 1.5"));
  delay(1500);
  if (! rtc.begin()) {
    lcd.print(F("Relógio ruim"));
    delay(10000);  
  }  
  if (rtc.lostPower()) {                 // lê status register do relógio. Se houve troca de bateria ...
    // lcd.println("Bateria nova!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }
// ==============================  C A R R E G A   V A L O R E S   D E   C A L I B R A Ç Ã O  =================================

  Vcaltemp = mem.readFloat(0);     // Lê calibração de tensão (4 bytes). Nota: A biblioteca considera double=8bytes, mas no Arduino minipro são 4, por isso o readFloat e não readDouble.
  if (Vcaltemp > 70 and Vcaltemp < 130){      // test to see if Vcal is reasonable.
    Vcal = Vcaltemp;
  }
  else {
    Vcal = 100.0;      // default if memory was empty/bad
    mem.writeFloat(0, Vcal);
  } 

  Icaltemp = mem.readFloat(4);     // Lê calibração de tensão (4 bytes). Nota: A biblioteca considera double=8bytes, mas no Arduino minipro são 4, por isso o readFloat e não readDouble.
  if (Icaltemp > 2 and Icaltemp < 20){      // test to see if Ical is reasonable.
    Ical = Icaltemp;   
  }
  else {
    Ical = 14;         // default if memory was empty/bad
    mem.writeFloat(4, Ical);
  }

  Fpcaltemp = mem.readFloat(8);     // Lê calibração de tensão (4 bytes). Nota: A biblioteca considera double=8bytes, mas no Arduino minipro são 4, por isso o readFloat e não readDouble.
  if (Fpcaltemp > 0 and Fpcaltemp < 3){      // test to see if Fpcal is reasonable.
    Fpcal = Fpcaltemp;
  }
  else {
    Fpcal = 1.3;         // default if memory was empty/bad
    mem.writeFloat(8, Fpcal);
  }  Fpcaltemp = mem.readFloat(8);     // Lê calibração de tensão (4 bytes). Nota: A biblioteca considera double=8bytes, mas no Arduino minipro são 4, por isso o readFloat e não readDouble.
  
  reaiskWhtemp = mem.readFloat(12);     // Lê calibração de tensão (4 bytes). Nota: A biblioteca considera double=8bytes, mas no Arduino minipro são 4, por isso o readFloat e não readDouble.
  if (reaiskWhtemp > 0 and reaiskWhtemp < 2){      // test to see if value for the kWh is reasonable.
    reaiskWh = reaiskWhtemp;
  }
  else {
    reaiskWh = 0.5948;         // default if memory was empty/bad
    mem.writeFloat(12, reaiskWh);
  }
  
  // initialize library with calibrated values
  emon1.voltage(2, Vcal, Fpcal);  // Voltage: input pin, calibration, phase_shift Gustavo = (2, 100.4, 1.4)
  emon1.current(1, Ical);       // Current: input pin, calibration. Gustavo = (1, 5.28)
  
// SD CARD setup ---------------------------------------------
lcd.clear();
if (SD.begin(chipSelect)) {
  lcd.print("SD Presente");     // card is present and accessible
  Grava = true;
  }
else {
  lcd.print("SD Ausente");     // card is present and accessible
  Grava = false;
}
delay(1500);
iCount = 0;
}  // END SETUP


// =======================================================    L O O P   ====================
void loop()
{
  emon1.calcVI(20,2000);         // Calculate all. (No.of half wavelengths (crossings), time-out)
  tensao   = emon1.Vrms;             //extract Vrms into Variable
  Irms     = emon1.Irms;             //extract Irms into Variable
  powerFActor   = emon1.powerFactor;      //extract Power Factor into Variable
  realPower     = emon1.realPower;        //extract Real Power into variable
  apparentPower = emon1.apparentPower;    //extract Apparent Power into variable
  //  emon1.serialprint();           // TEST: Print out all variables (realpower, apparent power, Vrms, Irms, power factor)

  tensaoMean = ((tensaoMean + tensao)/2);           // Vrms Médio
  IrmsMean = ((IrmsMean + Irms)/2);             // Irms Médio
  powerFActorMean = ((powerFActorMean + powerFActor)/2);      // Fator de Potência Médio
  realPowerMean = ((realPowerMean + realPower)/2);        // Pot. Ativa Média
  apparentPowerMean = ((apparentPowerMean + apparentPower)/2);    // Pot. Aparente Média
  iCount++;
  
  // tratamento botão frontal = MENU
  BotaoAgora = digitalRead(BOTAO);   // Botao pressionado = 0 (LOW), botao solto = 1 (HIGH)
   // Se a entrada foi de zero para 1.
  if (BotaoAgora == LOW && BotaoAntes == HIGH) {  
    if (!subMenu) {              // se não está em submenu
      if (menu == 8) {           // menu = o que vai aparecer no LCD 
        menu = 0;
        }
      else {
        menu++;
        if (menu > 5) {           // menus com submenus: vai para submenu depois de alguns segundos
          timeout1 = 20;           // tempo (x 0,5s) até ir para submenu
          }
      }
    }
    else {
      pressionou++;
    }
  }
  BotaoAntes = BotaoAgora;    // atualiza estado do botao
  // FIM tratamento botão frontal = MENU

  TimeStamp();  // carrega variáveis com o data/hora atual
  
  if (millis() - lastmillis > 512) {     // atualiza LCD a cada X milisegundos
    lastmillis = millis();    // memoriza tempo
    if (timeout1) timeout1--;
    if (timeout2) timeout2--;
    if (timeout5s) {
      timeout5s--;
      }
    else {
      h = dht.readHumidity();    //Le o valor da umidade
      t = dht.readTemperature(); //Le o valor da temperatura
      //========================= S E R I A L   O U T P U T   ( B L U E T O O T H  or S E R I A L  M O N I T O R)
      if (BlueTooth){
        Serial.print("E");
        Serial.print(realPowerMean,0);
        Serial.print(svirgula);
        Serial.print(apparentPowerMean,0);
        Serial.print(svirgula);
        Serial.print(IrmsMean,2);
        Serial.print(svirgula);
        Serial.print(tensaoMean,1);
        Serial.print(svirgula);
        Serial.print(powerFActorMean,2);
        Serial.print(svirgula);
        Serial.print(AcumWatt,1);
        Serial.print(svirgula);
        Serial.print(t,1);
        Serial.print(svirgula);
        Serial.print(h,1);
        Serial.print(svirgula);
        Serial.print(sTimeStampD);
        Serial.print(svirgula);
        Serial.print(sTimeStampT);
        Serial.print(svirgula);
        Serial.println(iCount, DEC);
        }
      timeout5s = 10;       // uma leitura a cada ~5 segundos. Não abaixe este valor abaixo de 4 (2s).
      }
    switch (menu) {
        case 0:
          {
          lcd.clear(); //Limpa o LCD e posiciona em 0,0
          lcd.print(F("Volt: "));
          lcd.print(tensao,1);
          lcd.print(F("V"));
          lcd.setCursor(0,1);
          lcd.print(F("Corr: "));
          lcd.print(Irms,2);
          lcd.print(F("A"));
          }
          break;
        
        case 1:
          {
          lcd.clear(); //Limpa o LCD e posiciona em 0,0
          lcd.print(F("FPot: "));
          lcd.print(powerFActor,2);
          lcd.setCursor(0,1);
          lcd.print(F("Corr: "));
          lcd.print(Irms,2);
          lcd.print(F("A"));
          }
          break;
  
        case 2:
          {
          lcd.clear(); //Limpa o LCD e posiciona em 0,0
          lcd.print(F("Real: "));
          lcd.print(realPower,1);
          lcd.print(F("W"));
          lcd.setCursor(0,1);
          lcd.print(F("Apar: "));
          lcd.print(apparentPower,1);
          lcd.print(F("VA"));
          }
          break;
  
        case 3:
          {
          lcd.clear();
          lcd.print(F("Temp: "));
          lcd.print(t,1);
          //Mostra o simbolo do grau quadrado mesmo(para fazer redondo usaria mais 64 bytes de memória de programa e 8 bytes de RAM)
          lcd.print((char)223);
          lcd.print(F("C"));
          lcd.setCursor(0,1);
          lcd.print(F("Umid: "));
          lcd.print(h,1);
          lcd.print("%");
          }
          break;
  
        case 4:
          {
          lcd.clear();
          lcd.print(sTimeStampD);  // Date now
          lcd.setCursor(0,1);
          lcd.print(sTimeStampT);  // Time now
          }
          break;
          
        case 5:
          {
          lcd.clear();
          lcd.print(F("Acumulado R$"));  // Valor acumulado desde a ultima partida
          lcd.setCursor(0,1);
          lcd.print(((reaiskWh*AcumWatt)/60000), 2);  // Valor em R$/kWh * watt / 60min por hora / 1000 W (variavel AcumWatt = W/minuto)
          }
          break;
        
  
        case 6:      // ============ C A L I B R A T I O N  ================
          {
          if (timeout1){                      // timeout 1st level menu
            lcd.clear();
            lcd.print(F("Calibracao"));  
            lcd.setCursor(0,1);
            lcd.print(F("entra em "));  // Time now
            lcd.print((timeout1/2));  // Time now
            if (timeout1 == 1) {           // time to press the button is running out !
              // inicializa para submenu
              Vcaltemp = Vcal;
              Icaltemp = Ical;
              Fpcaltemp = Fpcal;
              timeout2 = 20;                    // timeout 2nd level menu
              subMenu=1;
              pressionou = 0;
              }
            }
          else{
            lcd.clear(); //Limpa o LCD e posiciona em 0,0
            switch (subMenu) {
              case 1:
                {
                lcd.print(F("Volt: "));
                lcd.print(((tensao/Vcal)*Vcaltemp),1);
                lcd.setCursor(0,1);
                lcd.print(F("Diminui"));
                if (pressionou){              // buttom was pressed
                  Vcaltemp = Vcaltemp - (pressionou * 0.1);  // decrease calibration factor by 0,1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 2:
                {
                lcd.print(F("Volt: "));
                lcd.print(((tensao/Vcal)*Vcaltemp),1);
                lcd.setCursor(0,1);
                lcd.print(F("Aumenta"));
                if (pressionou){              // buttom was pressed
                  Vcaltemp = Vcaltemp + (pressionou * 0.1);  // increase calibration factor by 0,1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 3:
                {
                lcd.print(F("Corr: "));
                lcd.print(((Irms/Ical)*Icaltemp),2);
                lcd.setCursor(0,1);
                lcd.print(F("Diminui"));
                if (pressionou){              // buttom was pressed
                  Icaltemp = Icaltemp - (pressionou * 0.01);  // decrease calibration factor by 0,1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 4:
                {
                lcd.print(F("Corr: "));
                lcd.print(((Irms/Ical)*Icaltemp),2);
                lcd.setCursor(0,1);
                lcd.print(F("Aumenta"));
                if (pressionou){              // buttom was pressed
                  Icaltemp = Icaltemp + (pressionou * 0.01);  // increase calibration factor by 0,1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 5:
                {
                lcd.print(F("FPot: "));
                lcd.print(((powerFActor/Fpcal)*Fpcaltemp),2);
                lcd.setCursor(0,1);
                lcd.print(F("Diminui"));
                if (pressionou){              // buttom was pressed
                  Fpcaltemp = Fpcaltemp - (pressionou * 0.01);  // decrease calibration factor by 0,01 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 6:
                {
                lcd.print(F("FPot: "));
                lcd.print(((powerFActor/Fpcal)*Fpcaltemp),2);
                lcd.setCursor(0,1);
                lcd.print(F("Aumenta"));
                if (pressionou){              // buttom was pressed
                  Fpcaltemp = Fpcaltemp + (pressionou * 0.01);  // increase calibration factor by 0,01 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
  
            } // End Switch (submenu)

            lcd.setCursor(14,1);
            lcd.print((timeout2/2));  // Countdown for all submenus
            
            if (!timeout2) {
              subMenu++;    // if time over, then next submenu (timeout 2nd level menu)
              timeout2 = 20;
              pressionou = 0;
              if (subMenu ==7) {            // if it was the last submenu: save new values and then go back to 1st level menu
                if (Vcal != Vcaltemp){      // if voltage was calibrated, save and update
                  Vcal = Vcaltemp;
                  mem.writeFloat(0, Vcal);
                  emon1.voltage(2, Vcal, Fpcal);  // Voltage: input pin, calibration, phase_shift
                }
                if (Ical != Icaltemp){      // if current was calibrated, save and update
                  Ical = Icaltemp;
                  mem.writeFloat(4, Ical);
                  emon1.current(1, Ical);       // Current: input pin, calibration.
                }
                if (Fpcal != Fpcaltemp){      // if current was calibrated, save and update
                  Fpcal = Fpcaltemp;
                  mem.writeFloat(8, Fpcal);
                  emon1.voltage(2, Vcal, Fpcal);  // Voltage: input pin, calibration, phase_shift
                  }
                menu = 0;
                subMenu = 0;   
                }
              }
            }
          }  // End case 6 do primeiro menu
          break;

        case 7:      // ============ D A T E   A N D   T I M E   A D J U S T  S U B M E N U ================
          {
          if (timeout1){                      // timeout 1st level menu
            lcd.clear();
            lcd.print(F("Ajuste Data/Hora"));  
            lcd.setCursor(0,1);
            lcd.print(F("entra em "));  // Time now
            lcd.print((timeout1/2));  // Time now
            if (timeout1 == 1) {           // time to press the button is running out !
              // inicializa para submenu
              ano = agora.year();
              mes = agora.month();
              dia = agora.day();
              hora = agora.hour();
              minu = agora.minute();
              timeout2 = 20;                    // timeout 2nd level menu
              subMenu=1;
              pressionou = 0;
              }
            }
          else{
            lcd.clear(); //Limpa o LCD e posiciona em 0,0
            switch (subMenu) {
              case 1:
                {
                lcd.print(dia); lcd.print(F("/")); lcd.print(mes); lcd.print(F("/")); lcd.print(ano);
                lcd.setCursor(0,1);
                lcd.print(F("Dia -"));
                if (pressionou){              // buttom was pressed
                  dia = dia - pressionou;  // decrease day by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 2:
                {
                lcd.print(dia); lcd.print(F("/")); lcd.print(mes); lcd.print(F("/")); lcd.print(ano);
                lcd.setCursor(0,1);
                lcd.print(F("Dia +"));
                if (pressionou){              // buttom was pressed
                  dia = dia + pressionou;  // increase day by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 3:
                {
                lcd.print(dia); lcd.print(F("/")); lcd.print(mes); lcd.print(F("/")); lcd.print(ano);
                lcd.setCursor(0,1);
                lcd.print(F("Mes -"));
                if (pressionou){              // buttom was pressed
                  mes = mes - pressionou;  // decrease month by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 4:
                {
                lcd.print(dia); lcd.print(F("/")); lcd.print(mes); lcd.print(F("/")); lcd.print(ano);
                lcd.setCursor(0,1);
                lcd.print(F("Mes +"));
                if (pressionou){              // buttom was pressed
                  mes = mes + pressionou;  // increase month by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 5:
                {
                lcd.print(dia); lcd.print(F("/")); lcd.print(mes); lcd.print(F("/")); lcd.print(ano);
                lcd.setCursor(0,1);
                lcd.print(F("Ano -"));
                if (pressionou){              // buttom was pressed
                  ano = ano - pressionou;  // decrease year by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 6:
                {
                lcd.print(dia); lcd.print(F("/")); lcd.print(mes); lcd.print(F("/")); lcd.print(ano);
                lcd.setCursor(0,1);
                lcd.print(F("Ano +"));
                if (pressionou){              // buttom was pressed
                  ano = ano + pressionou;  // decrease year by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
                
              case 7:
                {
                lcd.print(hora); lcd.print(F(":")); lcd.print(minu); lcd.print(F(":00"));
                lcd.setCursor(0,1);
                lcd.print(F("Hora -"));
                if (pressionou){              // buttom was pressed
                  hora = hora - pressionou;  // decrease hour by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 8:
                {
                lcd.print(hora); lcd.print(F(":")); lcd.print(minu); lcd.print(F(":00"));
                lcd.setCursor(0,1);
                lcd.print(F("Hora +"));
                if (pressionou){              // buttom was pressed
                  hora = hora + pressionou;  // increase hour by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 9:
                {
                lcd.print(hora); lcd.print(F(":")); lcd.print(minu); lcd.print(F(":00"));
                lcd.setCursor(0,1);
                lcd.print(F("Minuto -"));
                if (pressionou){              // buttom was pressed
                  minu = minu - pressionou;  // decrease year by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 10:
                {
                lcd.print(hora); lcd.print(F(":")); lcd.print(minu); lcd.print(F(":00"));
                lcd.setCursor(0,1);
                lcd.print(F("Minuto +"));
                if (pressionou){              // buttom was pressed
                  minu = minu + pressionou;  // decrease year by 1 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
                                
            } // End switch submenu (time adjust)
            lcd.setCursor(14,1);
            lcd.print((timeout2/2));  // Countdown for all submenus
            
            if (!timeout2) {
              subMenu++;    // if time over, then next submenu (timeout 2nd level menu)
              timeout2 = 20;
              pressionou = 0;
              if (subMenu == 11) {            // if it was the last submenu: save new values and then go back to 1st level menu
                rtc.adjust(DateTime(ano, mes, dia, hora, minu, 0));     // set the time on DS3231
                menu = 4;
                subMenu = 0;   
                }
              }
            } // else
          }  // End case 6 do primeiro menu
          break;


        case 8:      // ============ A J U S T E   M O N E T Á R I O ================
          {
          if (timeout1){                      // timeout 1st level menu
            lcd.clear();
            lcd.print(F("Ajuste Monetario"));  
            lcd.setCursor(0,1);
            lcd.print(F("entra em "));  // Time now
            lcd.print((timeout1/2));  // Time now
            if (timeout1 == 1) {           // time to press the button is running out !
              // inicializa para submenu
              reaiskWhtemp = reaiskWh;
              timeout2 = 20;                    // timeout 2nd level menu
              subMenu=1;
              pressionou = 0;
              }
            }
          else{
            lcd.clear(); //Limpa o LCD e posiciona em 0,0
            switch (subMenu) {
              case 1:
                {
                lcd.print(F("Val.kWh: "));
                lcd.print(reaiskWhtemp,4);
                lcd.setCursor(0,1);
                lcd.print(F("Valor -"));
                if (pressionou){              // buttom was pressed
                  reaiskWhtemp = reaiskWhtemp - (pressionou * 0.0001);  // decrease day by 0.01 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
            
              case 2:
                {
                lcd.print(F("Val.kWh: "));
                lcd.print(reaiskWhtemp,4);
                lcd.setCursor(0,1);
                lcd.print(F("Valor +"));
                if (pressionou){              // buttom was pressed
                  reaiskWhtemp = reaiskWhtemp + (pressionou * 0.0001);  // decrease day by 0.01 for each press
                  timeout2 = 20;              // reset timeout for this submenu
                  pressionou = 0;
                  }
                }
                break;
                                
            } // End switch submenu (ajuste monetário)
            lcd.setCursor(14,1);
            lcd.print((timeout2/2));  // Countdown for all submenus
            
            if (!timeout2) {
              subMenu++;    // if time over, then next submenu (timeout 2nd level menu)
              timeout2 = 20;
              pressionou = 0;
              if (subMenu == 3) {            // if it was the last submenu: save new values and then go back to 1st level menu
                if (reaiskWh != reaiskWhtemp){      // if voltage was calibrated, save and update
                  reaiskWh = reaiskWhtemp;
                  mem.writeFloat(12, reaiskWh);
                }
                menu = 5;
                subMenu = 0;   
                }
              }
            } // else
          }  // End case 7 do primeiro menu
          break;

      } // End Switch Menu principal
    
    if (agora.second() == 0 and Grava){         // este if só roda uma vez por minuto
      if (Grava){         // este if só roda uma vez por minuto
        if (!logfile) {   // if a file is not already open
          
          // Build the file name to check if it exists in the SD card
          TimeStamp();                                       // load variables (used bellow) with the real time now
          sData = String(agora.year());                  // year
          if (agora.month() < 10) { sData+= "0"; }
          sData+= String(agora.month());               // month
          if (agora.day() < 10){ sData += "0"; }      
          sData+= String(agora.day()) + ".csv";          // monthday
          
          char cFilename[13];                      // transfer from string to char array for SD library
          sData.toCharArray(cFilename, 13);        // File name YYYYMMDD.csv
          
          if (! SD.exists(cFilename)) {                  // if there is no current file for this day, then create one.
            // Crete a new file and add a header.
            logfile = SD.open(cFilename, O_CREAT | O_APPEND | O_WRITE); 
            delay(700);
            if (logfile) {
                logfile.println(F("Data;Hora;Tensão;Corrente;Pot.Aparente;Fator de Potência;Pot.Ativa;Temp Celsius;Umidade;Leituras"));    
            }
          }
          else {
            // file is already there, just open for append another line.
            logfile = SD.open(cFilename, O_CREAT | O_APPEND | O_WRITE); 
          }
        } // End if (!logfile)
        
        lcd.clear();     // could open the file? Inform the user briefly(LCD)
        if (logfile) {
          logfile.print(sTimeStampD);
          logfile.print(svirgula);
          logfile.print(sTimeStampT);
          logfile.print(svirgula);
          logfile.print(tensaoMean, 1);
          logfile.print(svirgula);
          logfile.print(IrmsMean, 1);
          logfile.print(svirgula);
          logfile.print(apparentPowerMean, 1);
          logfile.print(svirgula);
          logfile.print(powerFActorMean, 2);
          logfile.print(svirgula);
          logfile.print(realPowerMean, 1);
          logfile.print(svirgula);
          logfile.print(t, 1);
          logfile.print(svirgula);
          logfile.print(h, 1);
          logfile.print(svirgula);
          logfile.println(iCount, DEC);
          iCount=0;   // reset contador de leituras por minutos
          lcd.print(F("Abriu arquivo"));    // Could open the file
        }
        else {
          lcd.print(F("Nao abriu arq."));   // Could not open file
        }
        bCount++;
        // Now we flush data to the Card! Don't sync too often - requires many bytes of I/O to SD card
        // which uses a lot of power (~200mA) and takes time
        if (bCount >= 5) {
          logfile.flush();
          bCount=0;  
        }
      }
      AcumWatt = AcumWatt + realPowerMean;   // Watt acumulator
      delay(1100);      // delay to prevent this part to be executed twice in a minute
      }
  }
}  // END "void loop"

// =================================================================== T i m e S t a m p
// Fills 2 TimeStamp variables with date and time (now)
void TimeStamp() {
      agora = rtc.now();                          // instância do objeto DateTime de nome "agora"
      sTimeStampD = "";  
      sTimeStampT = "";
      if (agora.day() < 10){ sTimeStampD = "0"; }      
      sTimeStampD+= String(agora.day()) + '/' ;        
      if (agora.month() < 10) { sTimeStampD+= "0"; }
      sTimeStampD+= String(agora.month())  + '/' + String(agora.year());      // Date: DD/MM/YYYY

      if (agora.hour() < 10){ sTimeStampT = "0"; }      
      sTimeStampT+= String(agora.hour()) + ':' ;        
      if (agora.minute() < 10) { sTimeStampT+= "0"; }
      sTimeStampT+= String(agora.minute())  + ':';
      if (agora.second() < 10) { sTimeStampT+= "0"; }
      sTimeStampT+= String(agora.second());   // Time: hh:mm:ss
} 
